-------------------------------------------------------------------------
--
--  vga_640x400x8.vhd
--
--	Author 	: Rene Bllhoff
--  Create 	: Sep. 2006
--
--  Purpose : 
--
--	High-Level-Komponente fr Generierung des VGA-Signals
--  - enthlt Komponente zum Ansteuern des SRAMs
--  - erzeugt VGA-Timing
--  - erzeugt VGA-RGB-Signale (ber Farb-Palette)
--  - erzeugt Maus-Zeiger
--
--  (c)2005-2006 by Rene Bllhoff
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity vga_640x400x8 is
    Port ( clk50m 		: in  	std_logic;
           res 				: in  	std_logic;

					 sram_adr		: out 	std_logic_vector (17 downto 0);
					 sram_data1	: inout std_logic_Vector (15 downto 0);
					 sram_data2	: inout std_logic_Vector (15 downto 0);
					 sram_cs1		:	out		std_logic;
					 sram_cs2		:	out		std_logic;
					 sram_ub1		:	out		std_logic;
					 sram_lb1		:	out		std_logic;
					 sram_ub2		:	out		std_logic;
					 sram_lb2		:	out		std_logic;
					 sram_we		:	out		std_logic;
					 sram_oe		:	out		std_logic;

					 videooff 	: in 		std_logic;
					 videostart : in 		std_logic_vector (19 downto 0);	

					 pal_idx		: in    std_logic_vector ( 1 downto 0);

					 mpos_x  		: in 		std_logic_vector ( 9 downto 0);
					 mpos_Y  		: in 		std_logic_vector ( 9 downto 0);
					 mouseidx 	: in		std_logic_vector ( 2 downto 0);
					 mouse_on	  : in 		std_logic;

					 gi_adr			: in 		std_logic_vector (19 downto 0);
					 gi_din			: in		std_logic_vector ( 7 downto 0);
					 gi_dout		: out 	std_logic_vector ( 7 downto 0);
					 gi_trg			:	in 		std_logic;
					 gi_rdy			:	out		std_logic;
					 gi_rw			:	in 		std_logic;
					 gi_req		 	:	out		std_logic;

					 briaddr 		: in 		std_logic_vector (10 downto 0);
					 briwdata 	: in 		std_logic_vector ( 7 downto 0);
					 brirdata1 	: out 	std_logic_vector ( 7 downto 0);
					 brirdata2 	: out 	std_logic_vector ( 7 downto 0);
					 briwen 		: in 		std_logic;
					 brien1			: in 		std_logic;
					 brien2			: in 		std_logic;

           vga_r  		: out 	std_logic_vector (3 downto 0);
           vga_g  		: out 	std_logic_vector (3 downto 0);
           vga_b  		: out 	std_logic_vector (3 downto 0);
           vga_vs 		: out 	std_logic;
           vga_hs 		: out 	std_logic
				 );
end vga_640x400x8;


architecture Behavioral of vga_640x400x8 is

	COMPONENT timing_gen640x400
	PORT(
		clkvga 			: IN std_logic;
		res 				: IN std_logic;      
		sync_tg			: in std_logic;    
		addr_out 		: OUT std_logic_vector(19 downto 0);
		hs_out 			: OUT std_logic;
		vs_out 			: OUT std_logic;
		hpos_out 		: OUT std_logic_vector(9 downto 0);
		vpos_out 		: OUT std_logic_vector(9 downto 0);
		pixoff_out 	: OUT std_logic
		);
	END COMPONENT;

	COMPONENT sram_access
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		ri_adr : IN std_logic_vector(19 downto 0);
		ri_din : IN std_logic_vector(7 downto 0);
		ri_rw : IN std_logic;
		ri_trg : IN std_logic;
		ri_radr : IN std_logic_vector(17 downto 0);    
		sri_data1 : INOUT std_logic_vector(15 downto 0);
		sri_data2 : INOUT std_logic_vector(15 downto 0);      
		sri_adr : OUT std_logic_vector(17 downto 0);
		sri_cs1 : OUT std_logic;
		sri_cs2 : OUT std_logic;
		sri_ub1 : OUT std_logic;
		sri_lb1 : OUT std_logic;
		sri_ub2 : OUT std_logic;
		sri_lb2 : OUT std_logic;
		sri_oe : OUT std_logic;
		sri_we : OUT std_logic;
		ri_dout : OUT std_logic_vector(7 downto 0);
		ri_rdy : OUT std_logic;
		ri_req : OUT std_logic;
		ri_rdata : OUT std_logic_vector(31 downto 0);
		ri_rda : OUT std_logic
		);
	END COMPONENT;


	COMPONENT vga_palette_ram
	PORT(
		clkpr : IN std_logic;
		clkpi : IN std_logic;

		praddr : IN std_logic_vector(9 downto 0);
		prdata : OUT std_logic_vector(15 downto 0);

		piaddr : IN std_logic_vector(10 downto 0);
		piwdata : IN std_logic_vector(7 downto 0);
		piwen : IN std_logic;
		pien : IN std_logic;          
		pirdata : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT vga_mouse_ram
	PORT(
		clkmr : IN std_logic;
		clkmi : IN std_logic;

		mraddr : IN std_logic_vector(12 downto 0);
		mrdata : OUT std_logic_vector(1 downto 0);

		miaddr : IN std_logic_vector(10 downto 0);
		miwdata : IN std_logic_vector(7 downto 0);
		miwen : IN std_logic;
		mien : IN std_logic;          
		mirdata : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;


	-- vga clock & timing gen (25mhz)
	signal clkvga 			: std_logic;

	signal graddr 			: std_logic_vector (19 downto 0);
	signal graddr_tg		: std_logic_vector (19 downto 0);
	signal grdata 			: std_logic_vector ( 7 downto 0);
	signal vsaddr_as		: std_logic_vector (19 downto 0);

	signal hs_out1 			: std_logic;
	signal vs_out1 			: std_logic;
	signal po_out1 			: std_logic;
	signal hs_pos_out1 	: std_logic_vector (9 downto 0);
	signal vs_pos_out1 	: std_logic_vector (9 downto 0);


	-- vga graphic interface blockram
	signal giaddr		:	std_logic_vector (12 downto 0);
	signal giwdata	:	std_logic_vector ( 7 downto 0);
	signal girdata	: std_logic_vector ( 7 downto 0);
	signal giwen		: std_logic;
	signal gien			: std_logic;

	signal videodata : std_logic_vector (31 downto 0);

	signal pixel_col : std_logic_vector ( 7 downto 0);
	signal praddr    : std_logic_vector ( 9 downto 0);
	signal rgb_value : std_logic_vector (15 downto 0);

	signal ri_rda	 	 : std_logic;

	signal mraddr_c	:	std_logic_vector ( 9 downto 0);
	signal mraddr_r	:	std_logic_vector (12 downto 0);
	signal mrdata	  :	std_logic_vector ( 1 downto 0);

	--signal mpos_x		: std_logic_vector ( 9 downto 0);
	--signal mpos_y		: std_logic_vector ( 9 downto 0);

	signal x_pos_is_mouse 	: std_logic;
	signal y_pos_is_mouse 	: std_logic;
	signal pos_is_mouse 		: std_logic;
	signal pos_is_mouse_z1	: std_logic;
	signal mouse_l_count		: std_logic_vector (4 downto 0);

	
begin

	-- komponenten einbinden

	Inst_timing_gen640x400: timing_gen640x400 PORT MAP(
		clkvga 			=> clkvga,
		res 				=> res,
		sync_tg			=> ri_rda,
		addr_out 		=> graddr_tg,
		hs_out 			=> hs_out1,
		vs_out 			=> vs_out1,
		hpos_out 		=> hs_pos_out1,
		vpos_out 		=> vs_pos_out1,
		pixoff_out 	=> po_out1
	);

	Inst_sram_access: sram_access PORT MAP(
		clk 				=> clk50m,
		res 				=> res,
		sri_adr 		=> sram_adr,
		sri_data1 	=> sram_data1,
		sri_data2 	=> sram_data2,
		sri_cs1 		=> sram_cs1,
		sri_cs2 		=> sram_cs2,
		sri_ub1 		=> sram_ub1,
		sri_lb1 		=> sram_lb1,
		sri_ub2 		=> sram_ub2,
		sri_lb2 		=> sram_lb2,
		sri_oe 			=> sram_oe,
		sri_we 			=> sram_we,
		ri_adr 			=> gi_adr,
		ri_din 			=> gi_din,
		ri_dout 		=> gi_dout,
		ri_rw 			=> gi_rw,
		ri_trg 			=> gi_trg,
		ri_rdy 			=> gi_rdy,
		ri_req 			=> gi_req,
		ri_radr 		=> graddr (19 downto 2),
		ri_rdata 		=> videodata,
		ri_rda 			=> ri_rda
	);

	Inst_vga_palette_ram: vga_palette_ram PORT MAP(
		clkpr 			=> clk50m,
		clkpi 			=> clk50m,
		praddr 			=> praddr,
		prdata 			=> rgb_value,
		piaddr 			=> briaddr,
		piwdata 		=> briwdata,
		pirdata 		=> brirdata1,
		piwen 			=> briwen,
		pien 				=> brien1
	);

	Inst_vga_mouse_ram: vga_mouse_ram PORT MAP(
		clkmr 			=> clk50m,
		clkmi 			=> clk50m,
		mraddr 			=> mraddr_r,
		mrdata 			=> mrdata,
		miaddr 			=> briaddr,
		miwdata 		=> briwdata,
		mirdata 		=> brirdata2,
		miwen 			=> briwen,
		mien 				=> brien2
	);


	-- takt erzeugung (noch berarbeiten ...)

	-- vga clock generator (25mhz)
	process (clk50m, res)
	begin
	  if res = '1' then 
		  clkvga <= '0';

		elsif clk50m'event and clk50m ='1' then
		  clkvga <= not clkvga;

		end if;
	end process;


	-- vga video start bernehmen
	process (clk50m, res)
	begin
	  if res = '1' then 
		  vsaddr_as <= (others => '0');

		elsif clk50m'event and clk50m ='1' then
			if vs_out1 = '1' then
				vsaddr_as <= videostart;
			end if;

		end if;
	end process;

	-- vga pixel adresse berechnen
	graddr <= graddr_tg + vsaddr_as;

	-- rgb pixel farb-adresse bauen
	praddr <= pal_idx & pixel_col;

	-- mouse-fenster signal bauen
	pos_is_mouse <= x_pos_is_mouse and y_pos_is_mouse and mouse_on;

	-- vga sync signale ausgeben
	vga_hs   <= hs_out1;
	vga_vs   <= vs_out1;


	-- mouse-fenster prfen
	process (hs_pos_out1, vs_pos_out1, mpos_x, mpos_y)
	begin
		-- pixoff_out (po_out1) MU geprft werden da h- und v-pos auch whrend des syncs ausgegeben werden
		if (po_out1 = '0') then
		  if (hs_pos_out1 >= mpos_x) and (hs_pos_out1 < (mpos_x + "0000100000")) then
			 	x_pos_is_mouse <= '1';
			else
			 	x_pos_is_mouse <= '0';
			end if;

		  if (vs_pos_out1 >= mpos_y) and (vs_pos_out1 < (mpos_y + "0000100000")) then
			 	y_pos_is_mouse <= '1';
			else
			 	y_pos_is_mouse <= '0';
			end if;
		end if;
	end process;

	-- mouse-ram zhler behandeln
	process (clk50m, res, clkvga, pos_is_mouse, vs_out1, mraddr_c)
	begin
		-- addresse (2 lsbs) "umdrehen" damit spiegelung der 2'er bit-gruppen (pixel sollen mit msb anfangen) 
		-- keine besonderen sw-kniffe braucht
		mraddr_r <= mouseidx & mraddr_c ( 9 downto 2) & not mraddr_c (1) & not mraddr_c (0);

		if res = '1' then
			mraddr_c 			<= (others => '0');
			mouse_l_count <= (others => '0');
		elsif clk50m'event and clk50m = '1' then

		  if clkvga = '1' then
				pos_is_mouse_z1 <= pos_is_mouse;

				if mouse_l_count /= "00000" then
				  mouse_l_count <= mouse_l_count 	+ '1';
					mraddr_c 			<= mraddr_c 			+ '1';
				end if; 

				if (pos_is_mouse_z1 = '0') and (pos_is_mouse = '1') then
					mouse_l_count <= "00001";
					mraddr_c 			<= mraddr_c + '1';
				end if;

				if vs_out1 = '0' then -- !! achtung sync ist bei std-auflsung 640x400 invertiert !! bei anderen auflsungen anpassen
					mraddr_c <= (others => '0');
				end if;
			end if;
		end if;
	end process;


	-- pixel farbe ermitteln
	process	(hs_pos_out1, videodata)
	begin
	  case hs_pos_out1 (1 downto 0) is
			when "11"  	=> pixel_col <= videodata (31 downto 24);
			when "10"  	=> pixel_col <= videodata (23 downto 16);
			when "01"  	=> pixel_col <= videodata (15 downto  8);
			when "00"  	=> pixel_col <= videodata ( 7 downto  0);
			when others => pixel_col <= (others => '0');
		end case;
	end process;

	-- pixel ausgeben
	process	(grdata, hs_pos_out1, po_out1)
	begin
	  if po_out1 = '0' and videooff = '0' then
		  if pos_is_mouse = '0' then
			  vga_r <= rgb_value ( 3 downto 0);
			  vga_g <= rgb_value ( 7 downto 4);
			  vga_b <= rgb_value (11 downto 8);
			else
				case mrdata is
				  when "00" 	=> vga_r <= X"0"; vga_g <= X"0"; vga_b <= X"0";
				  when "01" 	=> vga_r <= X"f"; vga_g <= X"f"; vga_b <= X"f";
				  when "10" 	=> vga_r <= rgb_value ( 3 downto 0);
												 vga_g <= rgb_value ( 7 downto 4);
												 vga_b <= rgb_value (11 downto 8);
					when others => vga_r <= X"0"; vga_g <= X"0"; vga_b <= X"0";
				end case;
			end if;
		else
			vga_r <= X"0";
			vga_g <= X"0";
			vga_b <= X"0";
		end if;
	end process;




end Behavioral;
